#! /usr/bin/env python
# -*- coding: utf8 -*-
#
# Markus Reinert, 28.10.2012
#
# Pong-Spiel

import pygame
import time

from Ball import *
from Bat  import *


def game_draw(ball, bats, wallsAt):
	window.fill(colorBg)
	# Ball
	pygame.draw.circle(window, colorBl, (int(ball.pos[0]*SCALE), int(ball.pos[1]*SCALE)), SCALE/2)
	# Schläger
	for bat in bats:
		if bat.line == 1:	pygame.draw.rect(window, colorBt, ((sizeField[0]-1)*SCALE, bat.pos*SCALE, SCALE, bat.sizeBat*SCALE))
		elif bat.line == 3:	pygame.draw.rect(window, colorBt, (0, bat.pos*SCALE, SCALE, bat.sizeBat*SCALE))
	# Wände
	pygame.draw.rect(window, colorWl, (SCALE, 0, (sizeField[0]-2)*SCALE, SCALE))
	pygame.draw.rect(window, colorWl, (SCALE, (sizeField[1]-1)*SCALE, (sizeField[0]-2)*SCALE, SCALE))
	if 1 in wallsAt: pygame.draw.rect(window, colorWl, ((sizeField[0]-1)*SCALE, SCALE, SCALE, (sizeField[1]-2)*SCALE))
	if 3 in wallsAt: pygame.draw.rect(window, colorWl, (0, SCALE, SCALE, (sizeField[1]-2)*SCALE))
	pygame.display.flip()

def game_event(bats):
	for event in pygame.event.get():
		if event.type == pygame.QUIT: 
			exit_()
		elif event.type == pygame.KEYDOWN:
			if event.key == pygame.K_ESCAPE: return "exit"
			for kkv in PMV:
				if event.key == kkv[0]:
					for bat in bats:
						if bat.keys == kkv[2]: bat.pressed = 1
				elif event.key == kkv[1]:
					for bat in bats:
						if bat.keys == kkv[2]: bat.pressed = -1
		elif event.type == pygame.KEYUP:
			for kkv in PMV:
				if event.key == kkv[0] or event.key == kkv[1]:
					for bat in bats:
						if bat.keys == kkv[2]: bat.pressed = 0

def game(players):
	# Schläger und Wände erzeugen
	batsAt = {}
	wallsAt = [0, 2]
	i = 0
	for player in players:
		if i%2 == 1:
			if typesKey[players[i]%6] != "WL":	batsAt[i] = typesKey[players[i]%6]
			else:								wallsAt.append(i)
		i += 1
	# Schläger erzeugen
	bats = []
	for line in batsAt:		bats.append(Bat(line, sizeField[1], sizeBat, batsAt.get(line)))
	# Ball erzeugen
	ball = Ball(vBall, sizeField, wallsAt, bats)
	timeStart = time.time()
	while True:
		for bat in bats:	bat.move(bat.pressed)
		game_draw(ball, bats, wallsAt)
		if game_event(bats) == "exit":	break
		method_ki(ball, bats)
		time.sleep(0.04)
		fly = ball.fly()
		if not fly[0]:
			print "Verloren!\nSpieler",
			# auswerten bei welchem Spieler
			pos = fly[1]
			if   pos[0] < 0:			print "links"
			elif pos[0] > sizeField[0]:	print "rechts"
			break
		timeNow = time.time()
		if timeNow - timeStart > 1:	timeStart = timeNow
	return "over"


def menuStart_draw(selected):
	window.fill(colorBg)
	# Text
	window.blit(textPlay, (40, 1*40+25))
	window.blit(textExit, (40, 2*40+25))
	# Schläger
	pygame.draw.rect(window, colorBt, (20, selected*40+20, 10, 30))
	pygame.display.flip()

def menuStart_event():
	for event in pygame.event.get():
		if event.type == pygame.QUIT:
			exit_()
		elif event.type == pygame.KEYDOWN:
			if event.key == pygame.K_ESCAPE:	exit_()
			elif event.key == pygame.K_DOWN:	return 1
			elif event.key == pygame.K_UP:		return -1
			elif event.key == pygame.K_RETURN:	return "enter"
		elif event.type == pygame.MOUSEMOTION:
			if  65 < event.pos[1] <  85: 		return "abs1"
			if 105 < event.pos[1] < 125: 		return "abs2"
		elif event.type == pygame.MOUSEBUTTONDOWN:
			if  65 < event.pos[1] <  85: 		return "enter1"
			if 105 < event.pos[1] < 125: 		return "enter2"
	return 0

def menuStart():
	# Start-Menü
	selected = 1
	while True:
		menuStart_draw(selected)
		rSelect = menuStart_event()
		if type(rSelect) == str:
			if rSelect.startswith("enter") and rSelect != "enter":
				selected = int(rSelect[5])
				rSelect = "enter"
			if rSelect == "enter":
				if selected == 1:	menuPlayer()
				elif selected == 2:	exit_()
			elif rSelect.startswith("abs"):		selected = int(rSelect[3])
			continue
		newSelect = selected + rSelect
		if newSelect >= 1 and newSelect <= 2:	selected = newSelect


def menuPlayer_draw(player, players):
	window.fill(colorBg)
	# Text
	textPlayer = \
		(\
		fontMenu.render(typesKey[players[1]%6], 1, colorTx),\
		fontMenu.render(typesKey[players[3]%6], 1, colorTx),\
		)
	coordsx = (sizeField[0]*SCALE-50, 25)
	coordy  =  sizeField[1]*SCALE/2-textPlayer[0].get_height()/2
	for coordx in coordsx:	window.blit(textPlayer[coordsx.index(coordx)], (coordx, coordy))
	# Start
	window.blit(textStart, coordStart)
	# Schläger
	pygame.draw.rect(window, colorBt, (coordsx[player]-20, coordy-5, 10, 30))
	# Wände
	pygame.draw.rect(window, colorWl, (SCALE, 0, (sizeField[0]-2)*SCALE, SCALE))
	pygame.draw.rect(window, colorWl, (SCALE, (sizeField[1]-1)*SCALE, (sizeField[0]-2)*SCALE, SCALE))
	
	pygame.display.flip()
	return coordsx, coordy

def menuPlayer_event((coordsx, coordy)):
	for event in pygame.event.get():
		if event.type == pygame.QUIT:
			exit_()
		elif event.type == pygame.KEYDOWN:
			if event.key == pygame.K_ESCAPE:		return "exit"
			elif event.key == pygame.K_RIGHT:		return 0
			elif event.key == pygame.K_LEFT:		return 1
			elif event.key == pygame.K_UP:			return "enter"
			elif event.key == pygame.K_DOWN:		return "minus"
			elif event.key == pygame.K_SPACE:		return "start"
			elif event.key == pygame.K_RETURN:		return "start"
		elif event.type == pygame.MOUSEMOTION:
			for coordx in coordsx:
				if coordx-20 < event.pos[0] < coordx+30:
					if coordy-10 < event.pos[1] < coordy+40: return coordsx.index(coordx)
		elif event.type == pygame.MOUSEBUTTONDOWN:
			if event.button == 1 or event.button == 5:	rText = "enter"
			if event.button == 3 or event.button == 4:	rText = "minus"
			for coordx in coordsx:
				if coordx-20 < event.pos[0] < coordx+30:
					if coordy-10 < event.pos[1] < coordy+40: return rText+str(coordsx.index(coordx))
			if coordStart[0] < event.pos[0] < coordStart[0]+sizeStart[0]:
				if coordStart[1] < event.pos[1] < coordStart[1]+sizeStart[1]: return "start"
	return ""

def menuPlayer():
	# Spieler-Auswahl-Menü
	player = 1
	players = [-1, 0, -1, 1]
	while True:
		rSelect = menuPlayer_event(menuPlayer_draw(player, players))		# _draw gibt Koordinaten zurück, werden an _event übergeben
		if type(rSelect) == str:
			if rSelect.startswith("enter") and rSelect != "enter":
				player = int(rSelect[5])
				rSelect = "enter"
			elif rSelect.startswith("minus") and rSelect != "minus":
				player = int(rSelect[5])
				rSelect = "minus"
			pl = player*2+1
			if rSelect == "enter":		players = players[:pl] + [players[pl]+1] + players[pl+1:]		# erhöht gewählten Wert um 1
			elif rSelect == "minus":	players = players[:pl] + [players[pl]-1] + players[pl+1:]		# verringert
			elif rSelect == "exit":			break
			elif rSelect == "start":
				if game(players) == "over":	break
			continue
		player = rSelect


def method_ki(ball, bats):
	for bat in bats:
		if bat.keys == "KI":
			# wenn Bewegung auf Schläger zu
			if bat.line == 1:
				if ball.velocity[0] >= 0: continue
			elif bat.line == 3:
				if ball.velocity[0] <= 0: continue
			posBall = ball.pos[1]
			if ball.velocity[1] < 0:		# y-Geschwindigkeit: Bewegung nach unten
				posBat0 = bat.pos+bat.sizeBat/2
				posBat1 = bat.pos+bat.sizeBat
			elif ball.velocity[1] > 0:		# Bewegung nach oben
				posBat0 = bat.pos+bat.sizeBat/2
				posBat1 = bat.pos+bat.sizeBat
			if posBall < posBat0:	bat.move(-1)
			elif posBall > posBat1:	bat.move(+1)

def exit_():
	exit()

PMV = \
	(\
	(pygame.K_DOWN, pygame.K_UP, "U/D"), \
	(pygame.K_RIGHT, pygame.K_LEFT, "L/R"), \
	(pygame.K_s, pygame.K_w, "W/S"), \
	(pygame.K_d, pygame.K_a, "A/D"), \
	(pygame.K_l, pygame.K_j, "J/L"), \
	(pygame.K_k, pygame.K_i, "I/K"), \
	(pygame.K_KP5, pygame.K_KP8, "8/5"), \
	(pygame.K_KP6, pygame.K_KP4, "4/6"), \
	)		# pygame.K_* um bat.pressed erhöhen (Plus), um bat.pressed verringern (Minus), benötigter bat.keys (Value)

typesKey = ["KI", "U/D", "W/S", "J/L", "8/5", "WL"]

# Einstellungen
SCALE = 15

sizeField = (50, 30)
sizeBat = 3

vBall = (-0.5, -0.2)

# Farben
colorBg = (0,	0,		0)		# Hintergrund
colorBl = (255,	0,		0)		# Ball
colorBt = (0,	255,	0)		# Bat = Schläger
colorWl = (255,	255,	0)		# Wall = Wände
colorTx = (255,	255,	255)	# Texte

# Grafik initialisiern
pygame.init()
window = pygame.display.set_mode((sizeField[0]*SCALE, sizeField[1]*SCALE))
pygame.display.set_caption("Pong2Py")

fontMenu	=	pygame.font.Font(None, 20)
fontStart	=	pygame.font.Font(None, 30)

textPlay	=	fontMenu. render("Spielen",	1, colorTx)
textExit	=	fontMenu. render("Beenden",	1, colorTx)
textStart	=	fontStart.render("START",	1, colorTx)

sizeStart	=	textStart.get_size()
coordStart	=	(sizeField[0]*SCALE/2-sizeStart[0]/2, sizeField[1]*SCALE/2-sizeStart[1]/2)

# Start
menuStart()
